import section from '../es-modules/distributed-systems/section-tests/x/index.js';
import assert from 'assert';
import StackTrace from '../StackTrace.js';



section('Stack-Parser', (section) => {
    section.test('Function', async () => {
        await (async () => {
            const a = function() {
                throw new Error('Function Fail!');
            };

            a();
        })().catch((err) => {
            const stack = new StackTrace().getStack(err);

            assert(stack);
            assert(stack.length > 2);
            assert(stack[0].source.endsWith('200.000-stack-parser.js'));
            assert.equal(stack[0].name, 'a');
            assert(stack[0].line > 0);
            assert(stack[0].line > 0);
        });
    });


    section.test('Promise.all', async () => {
        await (async () => {
            const array = Array.apply(null, { length: 10 }).map(async (v, i) => {
                if (i === 4) throw new Error('Prmise.all Fail!');
            });

            await Promise.all(array);
        })().catch((err) => {
            const stack = new StackTrace().getStack(err);

            assert(stack);
            assert(stack.length > 2);
            assert(stack[0].source.endsWith('200.000-stack-parser.js'));
            assert.equal(stack[1].name, 'Array.map');
            assert(stack[0].line > 0);
            assert(stack[0].line > 0);
        });
    });


    section.test('new Class', async () => {
        await (async () => {
            const Cls = class {
                constructor() {
                    throw new Error('Constructor Fail!');
                }
            };

            new Cls();
        })().catch((err) => {
            const stack = new StackTrace().getStack(err);

            assert(stack);
            assert(stack.length > 2);
            assert(stack[0].source.endsWith('200.000-stack-parser.js'));
            assert.equal(stack[0].name, 'new Cls');
            assert(stack[0].line > 0);
            assert(stack[0].line > 0);
        });
    });


    section.test('Class Method', async () => {
        await (async () => {
            const Cls = class {
                doThings() {
                    throw new Error('donThings Fail!');
                }
            };

            new Cls().doThings();
        })().catch((err) => {
            const stack = new StackTrace().getStack(err);

            assert(stack);
            assert(stack.length > 2);
            assert(stack[0].source.endsWith('200.000-stack-parser.js'));
            assert.equal(stack[0].name, 'Cls.doThings');
            assert(stack[0].line > 0);
            assert(stack[0].line > 0);
        });
    });


    section.test('Eval', async () => {
        await (async () => {
            eval(`
                eval("throw new Error('eval Fail!');");
            `);
        })().catch((err) => {
            const stack = new StackTrace().getStack(err);

            assert(stack);
            assert(stack.length > 2);
            assert(stack[0].source.endsWith('200.000-stack-parser.js'));
            assert.equal(stack[0].name, 'eval');
            assert(stack[0].line > 0);
            assert(stack[0].line > 0);
        });
    });


    section.test('getStack using a custom built error object', async () => {
        const err = {
            name: 'Error',
            message: "Failed to resolve vraiable 'SERVICE_REGISTRY_DB_PASSWORD' on the config key 'database.hosts.0.password'!",
            stack: `Error: Failed to resolve vraiable 'SERVICE_REGISTRY_DB_PASSWORD' on the config key 'database.hosts.0.password
    at RainbowConfig.replaceSecrets (/node_modules/@rainbow-industries/rainbow-config/npm/src/RainbowConfig.js:133:23)
    at RainbowConfig.replaceSecrets (/node_modules/@rainbow-industries/rainbow-config/npm/src/RainbowConfig.js:97:22)
    at RainbowConfig.replaceSecrets (/node_modules/@rainbow-industries/rainbow-config/npm/src/RainbowConfig.js:97:22)
    at RainbowConfig.replaceSecrets (/node_modules/@rainbow-industries/rainbow-config/npm/src/RainbowConfig.js:97:22)
    at RainbowConfig.replaceSecrets (/node_modules/@rainbow-industries/rainbow-config/npm/src/RainbowConfig.js:97:22)
    at RainbowConfig.load (/node_modules/@rainbow-industries/rainbow-config/npm/src/RainbowConfig.js:87:14)
    at async ServiceRegistry.loadConfig (/node_modules/@infect/rda-service/src/Service.js:317:9)
    at async ServiceRegistry.initialize (/node_modules/@infect/rda-service/src/Service.js:117:13)
    at async ServiceRegistry.load (/node_modules/@infect/rda-service-registry/src/Service.js:33:9)
    at async ChildProcess.startService (/src/ChildProcess.js:56:9)`,
        };

        const stack = new StackTrace().getStack(err);
        console.log(stack)
        assert(stack);
        assert(stack.length > 8);
        assert(stack[0].source.endsWith('src/RainbowConfig.js'));
    });
});
